@@ -46,7 +46,7 @@ SCons.Warnings.enableWarningClass(NanopbWarning)
 
 def _detect_nanopb(env):
     '''Find the path to nanopb root directory.'''
-    if env.has_key('NANOPB'):
+    if 'NANOPB' in env:
         # Use nanopb dir given by user
         return env['NANOPB']
 
@@ -60,7 +60,7 @@ def _detect_nanopb(env):
 
 def _detect_python(env):
     '''Find Python executable to use.'''
-    if env.has_key('PYTHON'):
+    if 'PYTHON' in env:
         return env['PYTHON']
 
     p = env.WhereIs('python3')
@@ -85,7 +85,7 @@ def _detect_nanopb_generator(env):
 
 def _detect_protoc(env):
     '''Find the path to the protoc compiler.'''
-    if env.has_key('PROTOC'):
+    if 'PROTOC' in env:
         # Use protoc defined by user
         return env['PROTOC']
 
@@ -113,7 +113,7 @@ def _detect_protoc(env):
 
 def _detect_protocflags(env):
     '''Find the options to use for protoc.'''
-    if env.has_key('PROTOCFLAGS'):
+    if 'PROTOCFLAGS' in env:
         return env['PROTOCFLAGS']
 
     p = _detect_protoc(env)
